//  FileDialog.cpp version 1.5
//  yudit package - Unicode Editor for the X Window System (and Linux) 
//
//  Author: gsinai@iname.com (Gaspar Sinai)
//  GNU Copyright (C) 1997,1998,1999  Gaspar Sinai
// 
//  yudit version 1.5  Copyright(C) 30 November, 1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.4  Copyright(C) 25 November, 1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.3  Copyright(C)  5 April,    1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.2  Copyright(C) 10 December, 1998, Tokyo Japan  Gaspar Sinai
//  yudit version 1.1  Copyright(C) 23 August,   1998, Tokyo Japan  Gaspar Sinai
//  yudit version 1.0  Copyright(C) 17 May,      1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.99 Copyright(C)  4 April,    1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.97 Copyright(C)  4 February, 1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.95 Copyright(C) 10 January,  1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.94 Copyright(C) 17 December, 1997, Tokyo Japan  Gaspar Sinai
//  yudit version 0.9 Copyright (C)  8 December, 1997, Tokyo Japan  Gaspar Sinai
//  yutex version 0.8 Copyright (C)  5 November, 1997, Tokyo Japan  Gaspar Sinai
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#include "FileDialog.h"
#include "Translate.h"
#include <Xm/Xm.h>
#include <Xm/DialogS.h>
#include <Xm/List.h>
#include <Xm/PushB.h>
#include <Xm/MessageB.h>
#include <Xm/Label.h>
#include <Xm/RowColumn.h>
#include <Xm/Form.h>
#include <Xm/TextF.h>
#include <Xm/ScrolledW.h>
#include <Xm/Frame.h>
#include <Xm/ToggleB.h>
#include "MYudit.h"
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <dirent.h>
#include <AString.h>
#include <strings.h>

#define MAX_DIR_LENGTH		4096
static AString initialDir;
static int showHidden = 0;


FileDialog::FileDialog (Widget parent, const char* titleIn)
{
	const char*	okString;
	const char*	cancelString;
	const char*	helpString;
	const char*	aString;
	toplevel = 0;
	dialog = 0;
	lastString = 0;
	hidden = 0;
	char		currDir[MAX_DIR_LENGTH+1];

	if (initialDir.isNull())
	{
		if (getcwd(currDir, MAX_DIR_LENGTH)==0)
		{
			cerr << "warn: can not get current directory.";
			strcpy (currDir, "/");
		}
		currDir[MAX_DIR_LENGTH]=0;
		initialDir = (const char*) currDir;
	}
	workingDir = (const char*) initialDir;

	toplevel = XtVaCreatePopupShell ("FileDialogShell",
			xmDialogShellWidgetClass, parent,
			XmNtitle, titleIn, 0);
	okString = Translate ("Select");
	cancelString = Translate ("CD To");
	helpString = Translate ("Dismiss");
	dialog = XtVaCreateWidget ("FileDialog",
			xmMessageBoxWidgetClass, toplevel,
			XmNdialogType,  XmDIALOG_MESSAGE,
			XmNautoUnmanage, False,
			XmNdialogStyle, XmDIALOG_PRIMARY_APPLICATION_MODAL,
			XtVaTypedArg, XmNokLabelString,
			XmRString,	okString, strlen (okString) + 1,
			XtVaTypedArg, XmNcancelLabelString,
			XmRString,	cancelString, strlen (cancelString) + 1,
			XtVaTypedArg, XmNhelpLabelString,
			XmRString,	helpString, strlen (helpString) + 1,
			XmNwidth,  400,
			XmNheight, 400,
			0);
	XtVaSetValues (dialog, XmNdefaultButton, 
		XmMessageBoxGetChild (dialog, XmDIALOG_CANCEL_BUTTON), 0);
	//XtUnmanageChild (XmMessageBoxGetChild (dialog,
	//		XmDIALOG_HELP_BUTTON));
	XtAddCallback (dialog, XmNokCallback,
		(XtCallbackProc) MYuditFileButtonCallback, this);
	XtAddCallback (dialog, XmNcancelCallback,
		(XtCallbackProc) MYuditFileButtonCallback, this);
	XtAddCallback (dialog, XmNhelpCallback,
		(XtCallbackProc) MYuditFileButtonCallback, this);

	topForm = XtVaCreateManagedWidget ("FileTopForm",
			xmFormWidgetClass, dialog, 0);

	buildOptionMenu ();

	textField = XtVaCreateManagedWidget ("FileText",
			xmTextFieldWidgetClass, topForm,
			XmNrightAttachment, XmATTACH_FORM,
			XmNleftPosition, 25,
			XmNleftAttachment, XmATTACH_POSITION,
			XmNbottomAttachment, XmATTACH_FORM,
			0);

	aString = Translate ("File:");
	textFieldLabel = XtVaCreateManagedWidget ("FileTextLabel",
			xmLabelWidgetClass, topForm,
			XmNleftAttachment, XmATTACH_FORM,
			XmNrightWidget, textField,
			XmNrightAttachment, XmATTACH_WIDGET,
			XmNtopWidget, textField,
			XmNtopAttachment,  XmATTACH_OPPOSITE_WIDGET,
			XmNbottomAttachment,  XmATTACH_FORM,
			XmNisAligned,   True,
			XmNalignment, XmALIGNMENT_BEGINNING,
			XtVaTypedArg, XmNlabelString,
			XmRString,	aString, strlen (aString) + 1,
			0);

	//
	// Directories
	//
	aString = Translate ("Directories:");
	dirListLabel = XtVaCreateManagedWidget ("DirLabel",
			xmLabelWidgetClass, topForm,
			XmNrightOffset, 10,
			XmNleftAttachment, XmATTACH_FORM,
			XmNrightPosition, 50,
			XmNrightAttachment, XmATTACH_POSITION,

			XmNtopPosition,  12,
			XmNtopAttachment,  XmATTACH_POSITION,
			XmNbottomPosition,	24,
			XmNbottomAttachment,  XmATTACH_POSITION,

			XmNisAligned,   True,
			XmNalignment, XmALIGNMENT_BEGINNING,
			XtVaTypedArg, XmNlabelString,
			XmRString,	aString, strlen (aString) + 1,
			0);

	dirListSW = XtVaCreateManagedWidget ("FileDirList",
			xmScrolledWindowWidgetClass, topForm, 0);
	dirList = XtVaCreateManagedWidget ("FileDirList",
			xmListWidgetClass, dirListSW, 0);
	XtVaSetValues (dirListSW, 
		XmNtopWidget,	dirListLabel,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNrightOffset, 10,
		XmNleftAttachment, XmATTACH_FORM,
		XmNrightPosition, 50,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNbottomWidget, textField,
		XmNbottomOffset, 10,
		XmNbottomAttachment, XmATTACH_WIDGET,
		//XmNvisibleItemCount, 10, 
		0);
	//
	// Files
	//
	aString = Translate ("Files:");
	fileListLabel = XtVaCreateManagedWidget ("FileLabel",
			xmLabelWidgetClass, topForm,
			XmNrightAttachment, XmATTACH_FORM,
			XmNleftOffset, 10,
			XmNleftPosition, 50,
			XmNleftAttachment, XmATTACH_POSITION,

			XmNtopPosition,	12,
			XmNtopAttachment,  XmATTACH_POSITION,
			XmNbottomPosition,	24,
			XmNbottomAttachment,  XmATTACH_POSITION,

			XmNbottomAttachment,  XmATTACH_POSITION,
			XmNalignment, XmALIGNMENT_BEGINNING,
			XtVaTypedArg, XmNlabelString,
			XmRString,	aString, strlen (aString) + 1,
			0);

	fileListSW = XtVaCreateManagedWidget ("FileFileList",
			xmScrolledWindowWidgetClass, topForm, 0);
	fileList = XtVaCreateManagedWidget ("FileFileList",
			xmListWidgetClass, fileListSW, 0);
	XtVaSetValues (fileListSW, 
		XmNtopWidget,	fileListLabel,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNrightAttachment, XmATTACH_FORM,
		XmNleftOffset, 10,
		XmNleftPosition, 50,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNbottomWidget, textField,
		XmNbottomOffset, 10,
		XmNbottomAttachment, XmATTACH_WIDGET,
		//XmNvisibleItemCount, 10, 
		0);

	XtAddCallback (fileList, XmNbrowseSelectionCallback, 
		(XtCallbackProc) MYuditFileSelectCallback, this);
	XtAddCallback (fileList, XmNdefaultActionCallback, 
		(XtCallbackProc) MYuditFileSelectCallback, this);
//	XtAddCallback (dirList, XmNdefaultActionCallback, 
//		(XtCallbackProc) MYuditFileSelectCallback, this);
	XtAddCallback (dirList, XmNbrowseSelectionCallback, 
		(XtCallbackProc) MYuditFileSelectCallback, this);

	// Build option menu
	

}

FileDialog::~FileDialog()
{
	if (lastString) XtFree (lastString);
}

FileDialog::FileDialogButton
FileDialog::getButton ()
{
	lastButton = CANCEL;
	rebuildList();
	XtManageChild (dialog);
	// This one does not worth a penny.
	XtVaSetValues (dialog, XmNinitialFocus, textField, 0);
	XmProcessTraversal (textField, XmTRAVERSE_CURRENT);
	XtPopup (toplevel, XtGrabNone);
	MWait4Confirm (dialog);
	return lastButton;
}

const char*
FileDialog::getString()
{
	// Get it from textfield here
	if (lastString) XtFree (lastString);
	lastString = XmTextFieldGetString(textField);
	if (lastString==0 || lastString[0] ==0) return 0;
	fullPathName = (const char*)  workingDir;
	if (strcmp ((const char*) workingDir, "/"))
	{
		fullPathName += "/";
	}
	fullPathName += (const char*) lastString;
	return ((const char*) fullPathName);
}

void
FileDialog::dirPathCheck()
{
	int	i;
	// See if it took us to root
	if (strcmp ((const char*) workingDir, "/")==0 
		&& menuSize>1)
	{
		XtVaSetValues (menu, XmNmenuHistory, 
			menuItems[0], 0);
		for (i=1; i<menuSize; i++)
		{
			XtDestroyWidget (menuItems[i]);
		}
		menuSize = 1;
	}
}

void
FileDialog::rebuildList ()
{
	const char* current;
	struct stat	buf;
	AStringList	dirListString;
	AStringList	fileListString;
	DIR*		dir=0;
	struct dirent* 	entry;
	XmString*	dirListXmStr;
	XmString*	fileListXmStr;
	XmString	xmString;
	int		dirSize;
	int		i;
	AString		pathname;
	int		pos;

	XmListDeleteAllItems (dirList);
	XmListDeleteAllItems (fileList);

	current = (const char*) workingDir;

	if (stat (current, &buf) != 0 || (dir=opendir (current))==0)
	{
		cerr << "yudit: Can not read '"
			<< current << "' directory.\n";
		workingDir = (const char*) "/";
		current = (const char*) "/";

		dirPathCheck ();
		dir=opendir (current);
	}
	if (dir == 0) return;

	// read the directory and fill in the boxes.
	while ((entry=readdir (dir))!=0)
	{
		pathname = (const char*) current;
		if (strcmp ((const char*) current, "/"))
		{
			pathname += "/";
		}
		pathname += entry->d_name;
		if (stat ((const char*) pathname, &buf) != 0 
			&& lstat ((const char*) pathname, &buf) != 0)
		{
			cerr << "Yudit: can not find '" << (const char*) pathname << "'\n";
			continue;
		}
		if (hidden ==0 && entry->d_name[0] == '.') continue;
		if (S_ISDIR( buf.st_mode))
		{
			if (strcmp (entry->d_name, "..") == 0) continue;
			if (strcmp (entry->d_name, ".") == 0) continue;
			dirListString.addCaseSorted ((const char*) entry->d_name);
		}
		else if (S_ISREG (buf.st_mode) || S_ISLNK (buf.st_mode))
		{
			fileListString.addCaseSorted ((const char*) entry->d_name);
		}
	}
	closedir (dir);
	dirListXmStr= 0;

	if (strcmp ((const char*) current, "/"))
	{
		xmString = XmStringCreateSimple ("..");
		XmListAddItemUnselected (dirList, xmString, -1);
		XmStringFree (xmString);
	}
	if (dirListString.getSize() != 0)
	{
		dirListXmStr = new XmString[dirListString.getSize()];
		dirSize = 0;
		for (i=0; i<dirListString.getSize(); i++)
		{
			dirListXmStr[dirSize] = 
				XmStringCreateSimple ((char*)dirListString.at(i));
			if (dirListXmStr[dirSize]) dirSize++;
		}
		if (dirSize > 0)
		{
			XmListAddItems (dirList,
				dirListXmStr,  dirSize, 0);
		}
		for (i=0; i<dirSize; i++)
		{
			XmStringFree (dirListXmStr[i]);
		}
		delete dirListXmStr;
	}

	fileListXmStr= 0;
	if (fileListString.getSize() != 0)
	{
		fileListXmStr = new XmString[fileListString.getSize()];
		dirSize = 0;
		pos = -1;
		for (i=0; i<fileListString.getSize(); i++)
		{
			fileListXmStr[dirSize] = 
				XmStringCreateSimple ((char*)fileListString.at(i));
			if (fileListXmStr[dirSize])
			{
				if (lastString!=0 && 
					strcasecmp (fileListString.at(i), 
						lastString) ==0)
				{
					pos = dirSize;
				}
				dirSize++;
			}
		}
		if (dirSize > 0)
		{
			XmListAddItems (fileList,
				fileListXmStr,  dirSize, 0);
		}
		for (i=0; i<dirSize; i++)
		{
			XmStringFree (fileListXmStr[i]);
		}
		delete fileListXmStr;
		if (pos>0)
		{
			XmListSelectPos (fileList, pos+1, False);
		}
	}
	initialDir = (const char*) workingDir;
}

void
MYuditFileButtonCallback (Widget w, FileDialog* dialog, 
	XmAnyCallbackStruct *cbs)
{
	DIR* 	dir;
	char*	newDir;
	AString tryDir;
	AString fullPath;
	int	len;
	int	start;
	int	i;

	dialog->lastButton=FileDialog::CANCEL;
	if (cbs->reason == XmCR_HELP)
	{
		dialog->lastButton=FileDialog::CANCEL;
		XtPopdown (dialog->toplevel);
		return;
	}
	if (cbs->reason == XmCR_OK)
	{
		dialog->lastButton=FileDialog::OK;
		XtPopdown (dialog->toplevel);
		return;
	}
	if (cbs->reason == XmCR_CANCEL)
	{
		// Try to re-read 
		dialog->lastButton=FileDialog::CANCEL;
		newDir = XmTextFieldGetString(dialog->textField);

		if (newDir==0 || newDir[0] == 0)
		{
			if (newDir) XtFree (newDir);
			dialog->rebuildList();
			return;
		}
		len = strlen (newDir);
		for (i=0; i<len; i++)
		{
			if (newDir[i] == '/' && newDir[i+1] == '/')
			{
				dialog->rebuildList();
				XtFree (newDir);
				return;
			}
		}
		if (len > 1 && newDir[len-1] == '/')
		{
			newDir[len-1] = 0;
			len--;
		}

		if (newDir[0] == '/')
		{
			fullPath = newDir;
			dir = opendir ((const char*) fullPath);
			if (dir==0)
			{
				cerr << "yudit: Can not cd to '" <<
					(const char*) fullPath << "'.\n";
				XtFree (newDir);
				return;
			}
			closedir (dir);

			// Delete all except '/' from option menu items
			dialog->workingDir = "/";
			dialog->dirPathCheck();
		}
		else
		{
			fullPath = (const char*) dialog->workingDir;
			if (strcmp ((const char*) fullPath, "/")) fullPath += "/";
			fullPath += newDir;
			dir = opendir ((const char*) fullPath);
			if (dir==0)
			{
				cerr << "yudit: Can not cd to '" <<
					(const char*) fullPath << "'.\n";
				XtFree (newDir);
				return;
			}
			closedir (dir);
		}

		dialog->workingDir = (const char*) fullPath;
		dialog->rebuildList();
		start = 0;
		for (i=0; i<len; i++)
		{
			if (newDir[i] == '/' && i==0)
			{
				start = i+1;
				continue;
			}
			if (newDir[i] == '/')
			{
				newDir[i] = 0;
				tryDir = (const char*) &newDir[start];
				tryDir += "/";
				dialog->addOptionMenuItem ((const char*) tryDir);
				start = i+1;
			}
		}
		if (start != i)
		{
			tryDir = (const char*) &newDir[start];
			tryDir += "/";
			dialog->addOptionMenuItem ((const char*) tryDir);
		}
		dialog->dirPathCheck();
		XtFree (newDir);
		return;

	}
}

void
MYuditFileSelectCallback (Widget w, FileDialog* dialog, 
				XmListCallbackStruct *cbs)
{
	char*	newDir;
	AString	tryDir;
	DIR*	dir;
	int	len;
	int	i;

	if (w==dialog->fileList)
	{
		if (dialog->lastString==0) XtFree (dialog->lastString);
		dialog->lastString = 0;
		XmStringGetLtoR (cbs->item, XmSTRING_DEFAULT_CHARSET, &dialog->lastString);
		if (dialog->lastString > 0)
		{
			XmTextFieldSetString (dialog->textField, 
				dialog->lastString);
		}
		// we need to pop down
		if (cbs->reason == XmCR_DEFAULT_ACTION)
		{
			dialog->lastButton=FileDialog::OK;
			XtPopdown (dialog->toplevel);
		}
	}
	else if (w==dialog->dirList)
	{
		newDir = 0;
		XmStringGetLtoR (cbs->item, XmSTRING_DEFAULT_CHARSET, &newDir);
		if (newDir==0) return;
		if (strcmp (newDir, "..")==0)
		{
			XtFree (newDir);
			// Sanity
			if (dialog->menuSize < 2)
			{
				cerr << "yudit: oops 1 MYuditFileSelectCallback\n";
				return;
			}

			len = strlen ((const char*) dialog->workingDir);
			newDir = new char[len+1];
			strcpy (newDir, (const char*) dialog->workingDir);

			// Root must remain
			for (i=len-1; i>=0; i--)
			{
				if (newDir[i] == '/')
				{
					newDir[(i==0) ? i+1 : i ]=0;
					break;
				}
			}
			XtVaSetValues (dialog->menu, XmNmenuHistory, 
					dialog->menuItems[dialog->menuSize-2], 
					0);
			XtDestroyWidget (dialog->menuItems[dialog->menuSize-1]);
			dialog->menuSize--;

			dialog->workingDir = (const char*) newDir;
			dialog->rebuildList();
			delete newDir;
			return;
		}

		tryDir = (const char*) dialog->workingDir;
		if (strcmp ((const char*)tryDir, "/")) tryDir += "/";
		tryDir += newDir;
		if ((dir=opendir ((const char*) tryDir))==0)
		{
			cerr << "yudit: Can not cd to '" <<
				(const char*) tryDir << "'.\n";
			return;
		}
		closedir (dir);
		dialog->workingDir = (const char*) tryDir;
		dialog->rebuildList();
		if (strcmp ((const char*) dialog->workingDir, "/"))
		{
			tryDir = (const char*) newDir;
			tryDir += "/";
			dialog->addOptionMenuItem ((const char*) tryDir);
		}
		XtFree (newDir);
	}
}

void
FileDialog::buildOptionMenu ()
{
	char*		newDir;
	int		len;
	int		i;
	int		start;
	AString		labelStr;
	const char*	aString;

	manager = XmCreatePulldownMenu(topForm, "FileChoiceManager", 0, 0);
	menuSize = 0;
	menuArraySize = 0;

	menu = XtVaCreateManagedWidget ("FileChoice",
		xmRowColumnWidgetClass, topForm,
		XmNrowColumnType, XmMENU_OPTION,
		XtVaTypedArg, XmNlabelString,
		XmRString, "", strlen ("") +1,
		XmNorientation, XmHORIZONTAL,
		XmNsubMenuId, manager,
		XmNleftPosition, 50,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNtopAttachment, XmATTACH_NONE,
		XmNrightAttachment, XmATTACH_FORM,
		XmNbottomPosition, 12,
		XmNbottomAttachment, XmATTACH_POSITION,
		0);

	menuItems = 0;

	
	aString = Translate ("Hidden Entries");
	hiddenTB = XtVaCreateManagedWidget ("FileHidden",
		xmToggleButtonWidgetClass, topForm,
		XtVaTypedArg, XmNlabelString,
		XmRString, aString, strlen(aString) + 1,
		XmNalignment, XmALIGNMENT_END,
		XmNtopAttachment, XmATTACH_FORM,
		XmNleftAttachment, XmATTACH_FORM,
		XmNbottomPosition, 12,
		XmNbottomAttachment, XmATTACH_POSITION,
		0);

	XtAddCallback (hiddenTB, XmNvalueChangedCallback,
		(XtCallbackProc) MYuditFilePressedCallback, this);

	XmToggleButtonSetState (hiddenTB, (showHidden) ? True : False, False);
	hidden = showHidden;

	aString = Translate ("Tree:");
	menuLabel = XtVaCreateManagedWidget ("FileChoiceLabel",
		xmLabelWidgetClass, topForm,
		XmNalignment, XmALIGNMENT_END,
		XtVaTypedArg, XmNlabelString,
		XmRString, aString, strlen (aString) + 1,
		XmNtopAttachment, XmATTACH_FORM,
		XmNleftWidget, hiddenTB,
		XmNleftAttachment, XmATTACH_WIDGET,
		XmNbottomPosition, 12,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNrightPosition, 50,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNtopWidget, menu,
		XmNtopAttachment, XmATTACH_OPPOSITE_WIDGET,
		0);

	// Add option menu buttons
	len = strlen ((const char*) workingDir);
	newDir = new char[len+1];
	strcpy (newDir, (const char*) workingDir);
	for (i=0, start=0; i<len; i++)
	{
		if (newDir[i] == '/')
		{
			newDir[i] = 0;
			labelStr = (const char*) (&newDir[start]);
			labelStr += "/";
			start = i+1;
			addOptionMenuItem ((const char*) labelStr);
		}
	}
	if (start!=i)
	{
		labelStr = (const char*) (&newDir[start]);
		labelStr += "/";
		addOptionMenuItem ((const char*) labelStr);
	}
	delete newDir;
}

void
FileDialog::addOptionMenuItem(const char* itemName)
{
	Widget*		newArray;
	int		newSize;

	if (menuSize>=menuArraySize)
	{
		newSize = (menuArraySize==0)? 1 : menuArraySize<<1;
		newArray = new Widget[newSize];
		CHECKNULL (newArray);
		if (menuSize!=0)
		{
			memcpy (newArray, menuItems, menuSize *sizeof (Widget));
			delete menuItems;
		}
		menuItems = newArray;
		menuArraySize = newSize;
	}
	menuItems[menuSize] = XtVaCreateManagedWidget (itemName,
			xmPushButtonWidgetClass, manager,
			XtVaTypedArg, XmNlabelString,
			XmRString, itemName, strlen (itemName) + 1,
			0);
	XtVaSetValues (menu, XmNmenuHistory, menuItems[menuSize], 0);

	XtAddCallback (menuItems[menuSize], XmNactivateCallback,
		(XtCallbackProc) MYuditFilePressedCallback, this);
	
	menuSize++;
}

//
// Menu gets shorter and shorter
//
void 
MYuditFilePressedCallback (Widget w, FileDialog* dialog, 
	XmAnyCallbackStruct *cbs)
{
	int				i;
	int				len;
	int				slash;
	char*				newDir;
	XmToggleButtonCallbackStruct*	tcbs;

	if (w == dialog->hiddenTB)
	{
		tcbs = (XmToggleButtonCallbackStruct*) cbs;
		dialog->hidden =  (tcbs->set) ? 1 : 0;
		showHidden =  dialog->hidden;
		dialog->rebuildList();
		return;
	}

	for (i=dialog->menuSize-1; i>=0; i--)
	{
		if (w == dialog->menuItems[i]) break;
		XtDestroyWidget (dialog->menuItems[i]);
	}
	if (dialog->menuSize ==  i+1)
	{
		dialog->rebuildList();
		return;
	}
	dialog->menuSize = i+1;
	len = strlen ((const char*) dialog->workingDir);
	newDir = new char[len+1];
	strcpy (newDir, (const char*) dialog->workingDir);
	slash = 0;
	for (i=0; i<len; i++)
	{
		if (newDir[i] == '/')
		{
			slash++;
			if (slash >= dialog->menuSize)
			{
				if (slash==1)
				{
					 newDir[i+1] = 0;
				}
				else
				{
					 newDir[i] = 0;
				}
				break;
			}
		}
	}

	dialog->workingDir = (const char*) newDir;
	delete newDir;
	dialog->rebuildList();
}
