//  MenuActions.cpp version 1.5
//  yudit package - Unicode Editor for the X Window System (and Linux) 
//
//  Author: gsinai@iname.com (Gaspar Sinai)
//  GNU Copyright (C) 1997,1998,1999  Gaspar Sinai
// 
//  yudit version 1.5  Copyright(C) 30 November, 1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.4  Copyright(C) 25 November, 1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.3  Copyright(C)  5 April,    1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.2  Copyright(C) 10 December, 1998, Tokyo Japan  Gaspar Sinai
//  yudit version 1.1  Copyright(C) 23 August,   1998, Tokyo Japan  Gaspar Sinai
//  yudit version 1.0  Copyright(C) 17 May,      1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.99 Copyright(C)  4 April,    1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.97 Copyright(C)  4 February, 1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.95 Copyright(C) 10 January,  1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.94 Copyright(C) 17 December, 1997, Tokyo Japan  Gaspar Sinai
//  yudit version 0.9 Copyright (C)  8 December, 1997, Tokyo Japan  Gaspar Sinai
//  yutex version 0.8 Copyright (C)  5 November, 1997, Tokyo Japan  Gaspar Sinai
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#include "MainWidget.h"
#include <Translate.h>
#include <UCommon.h>
#include <UFontFreeType.h>
#include "MYudit.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>

#define LINE_LENGTH     4096

//
// BUG: menuBar->clickOn does not work because we are in a callback.
// Workaround: set up a workproc to do the job - later.
//
void
MainWidget::clickedOn (const MenuBarAction::MenuId menuIn, 
	const char* stringIn, const MenuBarAction::MenuState onOff)
{
	const char*		newVle;
	AString			fileNameIn;
	const char*		gotFileName;
	Dialog::Button		button=Dialog::CANCEL;
	Config::CheckList	type;
	MainWidget*		next;
	int			status;
	char*			filePathName=0;
	AString			errorMessage;
	PrintDialog::Button	printButton;
	const char*		gotErrors;
	Dialog*			promptDialog;

	newVle	= stringIn;

//	cerr << "info: clicked menu " << menuIn << " string: " 
//		<< stringIn << " " 
//		<< ((onOff==MenuBarAction::UON)? "ON" : "OFF") << "\n";

	if (onOff==MenuBarAction::UOFF) return;

	switch (menuIn)
	{
	case MenuBarAction::UFILE:
		if (strcmp (stringIn, "New Window") == 0)
		{
			(void) new MainWidget (name, parent, config);
		}
		if (strcmp (stringIn, "Close") == 0)
		{
			if (editArea->uIsModified())
			{
				if (ask (Translate ("Buffer has been modified.\nDo you really want to quit?")) == NO)
				{
					return;
				}
			}
			else if (getInstances() == 1)
			{
				if (ask (Translate ("Do you really want to exit?"))
					== NO )
				{
					return;
				}
			}
			delete this;
			return;
		}
		if (strcmp (stringIn, "Exit All") == 0)
		{
			if (ask (Translate ("Do you really want to exit?"))
				== NO )
			{
				return;
			}
			while ((next = getFirst())!=0)
			{
				delete next;
			}
			delete this;
			return;
		}
		if (strcmp (stringIn, "Save") ==0)
		{
			if (isEditable != UWidget::EDITABLE)
			{
				warn (Translate ("Buffer is not editable.\nTry 'Save As...'"));
				return;
			}
			if (fileName.isNull())
			{
				warn (Translate ("Buffer has no name.\nTry 'Save As...'"));
				return;
			}
			if (checkEncoding ()== 0) return;
			editArea->uSetCursor (UCursor::WAIT, 1);
			saveFile ((const char*) fileName);
			editArea->uSetCursor (UCursor::INSERT, 1);
			if (strncmp ((const char*) fileName, "file:",
				strlen("file:"))==0
				&& strcmp (&fileName[strlen("file:")], 
					config.getConfigPath()) == 0)
			{
				gotErrors=config.testConfig ("yuditrc");
				if (gotErrors==0) return;
				errorMessage= (const char*) Translate (
					"The yuditrc config file you\n"
					"have just updated has errors.\n"
					"Do not quit until you fix them.\n\n");
				errorMessage+=gotErrors;
				warn ((const char*) errorMessage);
				return;
			}
			encoding = (const char*) config.getChecked (Config::ENCODING);
			return;
		}
		if (strcmp (stringIn, "Save As...") == 0)
		{
			if (checkEncoding ()== 0) return;
			button = Dialog::Button (	
				Dialog::OK |Dialog::CANCEL);

			if (saveDialog==0) saveDialog = new FileDialog (
				mainWidget, "Yudit: Save");
			if (saveDialog->getButton ()==FileDialog::CANCEL)
			{
				return;
			}

			gotFileName = saveDialog->getString ();
			if (gotFileName==0||gotFileName[0]==0)
			{
				warn (Translate ("No file specified."));
				return;
			}
			fileNameIn="file:";
			fileNameIn+=gotFileName;

			editArea->uSetCursor (UCursor::WAIT, 1);
			status = saveFile ((const char*) fileNameIn);
			editArea->uSetCursor (UCursor::INSERT, 1);
			fileName = (const char*) fileNameIn; 
			XtVaSetValues (mainWidget, XmNtitle, 
				(const char*) fileNameIn, 0);
			encoding = (const char*) config.getChecked (Config::ENCODING);
			editArea->uSetEditable (UWidget::EDITABLE);
			isEditable = UWidget::EDITABLE;
			return;
		}
		if (strcmp (stringIn, "Open...") == 0)
		{
			if (editArea->uIsModified())
			{
				if (ask (Translate ("Buffer has been modified.\nDiscard changes?")) == NO)
				{
					return;
				}
			}
			button = Dialog::Button (	
				Dialog::OK |Dialog::CANCEL);

			if (loadDialog==0) loadDialog = new FileDialog (
				mainWidget, "Yudit: Open");
			if (loadDialog->getButton ()==FileDialog::CANCEL)
			{
				return;
			}
			gotFileName = loadDialog->getString ();
			if (gotFileName==0||gotFileName[0]==0)
			{
				warn (Translate ("No file specified."));
				return;
			}
			fileNameIn="file:";
			fileNameIn+=gotFileName;

			editArea->uSetCursor (UCursor::WAIT, 1);
			isEditable=UWidget::EDITABLE;
			editArea->uSetEditable (isEditable);
			loadFile ((const char*)fileNameIn);
			XtVaSetValues (mainWidget, XmNtitle, fileName.isNull()
				? MYuditTitle() : (const char*)fileName, 0);
			editArea->uSetCursor (UCursor::INSERT, 1);
			encoding = (const char*) config.getChecked (Config::ENCODING);
			return;
		}
		if (strcmp (stringIn, "Re-Load") ==0)
		{
			if (fileName.isNull())
			{
				warn (Translate ("Buffer has no name.\nTry 'Save As...' first."));
				return;
			}
			if (editArea->uIsModified())
			{
				if (ask (Translate ("Buffer has been modified.\nDiscard changes?")) == NO)
				{
					return;
				}
			}
			if (checkEncoding ()== 0) return;
			if (!fileName.isNull())
			{
				fileNameIn= (const char*) fileName;
				editArea->uSetEditable (UWidget::EDITABLE);
				loadFile ((const char*) fileNameIn);
			}
			editArea->uSetEditable (isEditable);
			encoding = (const char*) config.getChecked (Config::ENCODING);
			return;
		}
		if (strcmp (stringIn, "Config") == 0)
		{
			(void) new MainWidget (name, parent, config, 
				config.getConfigPath(), "UTF8", UWidget::EDITABLE);
			return;
		}
		if (strcmp (stringIn, "Print") == 0)
		{
			if (printDialog == 0)
			{
				printDialog = new PrintDialog (mainWidget,
					&config, editArea);
			}
			printButton = printDialog->print(
				editArea->uGetTextBuffer(),
				editArea->uGetTextSizes(),
				editArea->uGetTextSize(),
				(const char*) fileName);
			if (printButton == PrintDialog::CANCEL)
			{
				//delete printDialog;
				//printDialog=0;
				return;
			}
			gotErrors = printDialog->getMessage();
			if (gotErrors==0)
			{
				errorMessage= (const char*) Translate (
					"Print failed.\n"
					"You might not have printing properly\n"
					"configured in your config file.");
				warn ((const char*) errorMessage);
				//delete printDialog;
				//printDialog=0;
				return;
			}
			inform (gotErrors);
			//delete printDialog;
			//printDialog=0;
			return;
		}
		if (strcmp (stringIn, "Append...") == 0)
		{
			if (isEditable != UWidget::EDITABLE)
			{
				warn (Translate ("Buffer is not editable.\nTry 'Load'"));
				return;
			}

			if (loadDialog==0) loadDialog = new FileDialog (
				mainWidget, "Yudit: Open");
			if (loadDialog->getButton ()==FileDialog::CANCEL)
			{
				return;
			}

			gotFileName = loadDialog->getString ();
			if (gotFileName==0||gotFileName[0]==0)
			{
				warn (Translate ("No file specified."));
				return;
			}
			fileNameIn="file:";
			fileNameIn+=gotFileName;

			editArea->uSetCursor (UCursor::WAIT, 1);
			appendFile ((const char*) fileNameIn);
			editArea->uSetCursor (UCursor::INSERT, 1);
			XtVaSetValues (mainWidget, XmNtitle, fileName.isNull()
				? MYuditTitle() : (const char*)fileName, 0);
			return;
		}
		return;
	case MenuBarAction::UENCODING:
		type = Config::ENCODING;
		if (editArea->setConversion(newVle) == UString::ERROR)
		{
			warn (Translate ("Can not set desired encoding."));
			setToggle ();
			return;
		}
		break;
	case MenuBarAction::UINPUT:
		type = Config::INPUT;
		if (strcasecmp (newVle, "Straight") ==0
			|| strcasecmp (newVle, "None")==0)
		{
			editArea->setInput (0);
		}
		else if (editArea->setInput(newVle) == UWidget::ERROR)
		{
			
			warn (Translate ("Can not set desired input map."));
			setToggle ();
			return;
		}
		break;
	case MenuBarAction::UXINPUT:
		type = Config::XINPUT;
		if (setXInput (newVle)!=0)
		{
			warn (Translate ("Can not set up XInput."));
			setToggle ();
			return;
		}
		break;
	case MenuBarAction::UFONTNAME:
		type = Config::FONTNAME;
		if (editArea->uSetFont (newVle) == UWidget::ERROR)
		{
			warn (Translate ("Can not set desired font."));
			setToggle ();
			return;
		}
		editArea->uSetFontSize (atoi((const char*)config.getChecked(Config::FONTSIZE)));
		editArea->uSetFontSlant ((const char*)config.getChecked(Config::FONTSLANT));
		editArea->uSetFontWeight ((const char*)config.getChecked(Config::FONTWEIGHT));
		editArea->uSetFontSpacing ((const char*)config.getChecked(Config::FONTSPACING));
		editArea->uSetFontAddStyle ((const char*)config.getChecked(Config::FONT_ADD_STYLE));
		editArea->uSetFontAvgWidth ((const char*)config.getChecked(Config::FONT_AVG_WIDTH));
		editArea->refresh();
		break;
	case MenuBarAction::UFONTWEIGHT:
		type = Config::FONTWEIGHT;
		editArea->uSetFontWeight (newVle);
		editArea->refresh();
		break;
	case MenuBarAction::UFONTSIZE:
		type = Config::FONTSIZE;
		editArea->uSetFontSize (atoi(newVle));
		editArea->refresh();
		break;
	case MenuBarAction::UFONTSLANT:
		type = Config::FONTSLANT;
		editArea->uSetFontSlant (newVle);
		editArea->refresh();
		break;
	case MenuBarAction::UFONTSPACING:
		type = Config::FONTSPACING;
		editArea->uSetFontSpacing (newVle);
		editArea->refresh();
		break;
	case MenuBarAction::UFONT_ADD_STYLE:
		type = Config::FONT_ADD_STYLE;
		editArea->uSetFontAddStyle (newVle);
		editArea->refresh();
		break;
	case MenuBarAction::UFONT_AVG_WIDTH:
		type = Config::FONT_AVG_WIDTH;
		editArea->uSetFontAvgWidth (newVle);
		editArea->refresh();
		break;
	case MenuBarAction::UTABSIZE:
		type = Config::TABSIZE;
		editArea->setTabSize (atoi(newVle));
		break;
	case MenuBarAction::UWINDOW_BACKGROUND:
		type = Config::WINDOW_BACKGROUND;
		editArea->setBackground (newVle);
		XtVaSetValues (editForm, XtVaTypedArg, XmNbackground, 
			XmRString, stringIn, strlen(stringIn) + 1, 0);
		break;
	case MenuBarAction::UWINDOW_FOREGROUND:
		type = Config::WINDOW_FOREGROUND;;
		editArea->setForeground (newVle);
		break;
	case MenuBarAction::UCURSOR_COLOR:
		type = Config::CURSOR_COLOR;
		editArea->setCursorColor (newVle);
		break;
	case MenuBarAction::USEARCH:
		if (strcmp (stringIn, "Find/Replace") == 0)
		{
			findDialog = new FindDialog (mainWidget, editArea, &config);
			findDialog->findString();
			delete findDialog;
			findDialog = 0;
			return;
		}
		if (strcmp (stringIn, "Go To") == 0)
		{
			button = Dialog::Button(Dialog::OK|Dialog::CANCEL);
			promptDialog = new Dialog (
				mainWidget, "Yudit: Go To", 
				button, Dialog::DPROMPT);

			promptDialog->setMessage (Translate ("Line number:"));
			button = promptDialog->getButton ();
			gotErrors = promptDialog->getString();
			if (gotErrors==0 || button==Dialog::CANCEL)
			{
				delete promptDialog;
				return;
			}
			editArea->uGoTo(atoi(gotErrors));
			delete promptDialog;
			return;
		}
		return;
	case MenuBarAction::UHELP:
		if (strcmp (stringIn, "About") == 0)
		{
			inform (UWidgetVersion());
			return;
		}
		fileNameIn = (const char*) stringIn;
		fileNameIn += ".help";
		filePathName = UFindFile (UGetMapPath (), fileNameIn);
		if (filePathName==0)
		{
			warn (Translate ("Can not find help file."));
			return;
		}
		(void) new MainWidget (name, parent, config, filePathName, 
			"UTF8", UWidget::VIEW);
		delete filePathName;
		return;
	default:
		return;
	}
	config.setChecked (type, newVle);
}

void
MainWidget::inform (const char* messageIn)
{
	Dialog*		dialog;

	dialog = new Dialog (
		mainWidget, "Yudit: Information", Dialog::OK, Dialog::DINFO);
	dialog->setMessage (messageIn);
	dialog->getButton ();
	delete dialog;
}

void
MainWidget::warn (const char* messageIn)
{
	Dialog*		dialog;

	dialog = new Dialog (
		mainWidget, "Yudit: Message", Dialog::OK );
	dialog->setMessage (messageIn);
	dialog->getButton ();
	delete dialog;
}

MainWidget::YesNo
MainWidget::ask (const char* messageIn)
{
	Dialog*		dialog;

	Dialog::Button	button;

	button = Dialog::Button(Dialog::OK|Dialog::CANCEL);
	dialog = new Dialog (
		mainWidget, "Yudit: Question", button, Dialog::DQUESTION);

	dialog->setMessage (messageIn);
	button = dialog->getButton ();
	delete dialog;
	return ((button==Dialog::OK) ? YES : NO);
}

int
MainWidget::checkEncoding ()
{
        char            buffer[255];
        char            format[255];
        const char*     enc=(const char*) config.getChecked(Config::ENCODING);

        if (fileName.isNull() || enc==0 || encoding.isNull())
        {
                return 1;
        }
        if (strcmp ((const char*) encoding, enc)==0)
        {
                return 1;
        }
        strcpy (format, Translate (
                "The last file transaction on this buffer used encoding '%s'.\n"
                "Now your encoding is '%s'.\n\nShall we proceed?"));
        sprintf (buffer, format, (const char*) encoding, enc);
        if (ask (buffer)==NO)
        {
                return 0;
        }
        return 1;
}

int
MainWidget::saveFile (const char* filenameIn)
{
	unsigned char*		text;
	int			len;
	ofstream*		file;
	int			error;

	if (strlen(filenameIn) > strlen ("file:") 
		&& strncmp (filenameIn, "file:", strlen ("file:")) == 0)
	{
		file = new ofstream (filenameIn + strlen ("file:"));
	}
	else
	{
		file = new ofstream (filenameIn);
	}
	if (file->rdbuf() == 0 || !file->rdbuf()->is_open())
	{
		editArea->uSetCursor (UCursor::INSERT, 1);
		warn (Translate ("Write failed."));
		file->close();
		delete file;
		return -1;
	}
	error=0;
	text = editArea->getBuffer(&len);
	if (text !=0)
	{
		if (UWriteBuffer(file, text, len) ==-1) error=1;
		delete text;
	}
	file->close();
	delete file;
	if (error)
	{
		editArea->uSetCursor (UCursor::INSERT, 1);
		warn (Translate ("Write failed."));
		return -1;
	}
	editArea->uClearModified ();
	return 1;
}

int
MainWidget::loadFile (const char* filenameIn)
{
	int	retVle;
	editArea->clearText();
	retVle =  appendFile (filenameIn);
	editArea->uClearModified ();
	if (retVle>=0)
	{
		fileName = (const char*) filenameIn;
		encoding = (const char*) config.getChecked (Config::ENCODING);
	}
	else
	{
		fileName = (const char*) 0;
	}
	return retVle;
}


int
MainWidget::appendFile (const char* filenameIn)
{
	int	ret_vle;
	ret_vle = appendFileBase (filenameIn);
	editArea->refresh();
	if (ret_vle>=0)
	{
		if (fileName.isNull ())
		{
			fileName = (const char*) filenameIn;
			encoding = (const char*) config.getChecked (Config::ENCODING);
		}
	}
	return ret_vle;
}

int
MainWidget::appendFileBase (const char* filenameIn)
{
	ifstream*	file;
	unsigned char* 	buffer;
	int		len;

	if (strlen(filenameIn) > strlen ("file:") 
		&& strncmp (filenameIn, "file:", strlen ("file:")) == 0)
	{
		file = new ifstream (filenameIn + strlen ("file:"));
	}
	else
	{
		file = new ifstream ((const char*) filenameIn);
	}
	if (file->rdbuf() == 0 || !file->rdbuf()->is_open())
	{
		cerr << "Yudit: can not open '" << 
			filenameIn << "' to read.\n";
		editArea->uSetCursor (UCursor::INSERT, 1);
		warn ("Can not read file.");
		file->close ();
		delete file;
		return -1;
	}
	buffer = UReadBuffer (file, &len);
	editArea->appendText (buffer, len);
	delete buffer;
	file->close ();
	delete file;
	return 1;
}

int
MainWidget::setXInput (const char* nameIn)
{
	const XInputMap*	xInputMap;

	xInputMap = config.getXInputMap (nameIn);
	if (xInputMap==0)
	{
		cerr << "error: can not get XInputMap.\n";
		return -1;
	}
	return editArea->uSetXInput (nameIn, 
			(const char*) xInputMap->list[XInputMap::TYPE],
			(const char*) xInputMap->list[XInputMap::STYLE],
			(const char*) xInputMap->list[XInputMap::TEXT_TYPE],
			(const char*) xInputMap->list[XInputMap::ENCODING],
			(const char*) xInputMap->list[XInputMap::FONTMAP],
			atof ((const char*)xInputMap->list[XInputMap::VERSION]));
}

